#pragma once

#include "NPMessageHandler.h"
#include "DataContext.h"
#include "UpdaterDocument.h"
#include "Command/NotifyWindowCommand.h"
#include "TrayIconHandler.h"

const UINT WM_HIDE_NOTIFYWINDOW = WM_USER+1234;
const int NOTIFYWINDOW_SHOW_TIME = 5;

namespace tipupdater
{
	class Effect
	{
	public:
		static void FadeIn(int alpha, HWND windowHandle)
		{
			BLENDFUNCTION bf;
			bf.BlendOp = AC_SRC_OVER;
			bf.BlendFlags = 0;
			bf.AlphaFormat = 0;
			bf.SourceConstantAlpha = alpha;

			UpdateLayeredWindow(windowHandle, NULL, NULL, NULL, NULL, NULL, NULL, &bf, ULW_ALPHA);	
		}
	};

	/**
	@class AbstractShowEffect
	@brief FadeInShowEffect  θ Class
	*/
	class NotifyWindowHandler : public CNMessageHandler
	{
	public:
		static NotifyWindowHandler* CreateInstance()
		{
			if( m_notifyWindow )
			{
				return m_notifyWindow;
			}

			m_notifyWindow = new NotifyWindowHandler;
			m_notifyWindow->Create();
			return m_notifyWindow;
		}

		static void DestroyInstance()
		{
			if (m_notifyWindow)
			{
				// [:2008-12-18] Memory Leak  :  Destroyϴ ڵ  ߰
				m_notifyWindow->Destroy();
				delete m_notifyWindow;
			}
			m_notifyWindow = NULL;
		}

	private:
		static NotifyWindowHandler* m_notifyWindow;

	private:
		NotifyWindowHandler()
		{
			m_DataContext	= NULL;
			m_Document		= NULL;
			m_rootwindow	= NULL;
			m_window		= NULL;
			m_mouseHover	= FALSE;
		}

		~NotifyWindowHandler()
		{
			if (m_window.IsWindow())
				m_window.DestroyWindow();
		}

		DECLARE_WND_CLASS(_T("NotifyWindowHandler")) 
		// If this macro is not specified in your
		// class, ATL will generate a class name

		BEGIN_MSG_MAP(NotifyWindowHandler)
			MESSAGE_HANDLER(WM_TIMER,				OnTimer)
			MESSAGE_HANDLER(WM_HIDE_NOTIFYWINDOW,	OnHide)
			MESSAGE_HANDLER(WM_MOUSEHOVER,			OnMouseHover)
			MESSAGE_HANDLER(WM_MOUSELEAVE,			OnMouseLeave)
		END_MSG_MAP()

	protected:
		LRESULT OnMouseHover(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
		{
			m_mouseHover = TRUE;
			m_timer_count = 25;
			return 1;
		}

		LRESULT OnMouseLeave(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
		{
			m_mouseHover = FALSE;
			return 1;
		}

	public:
		void SetDocument( UpdaterDocument *doc )
		{
			m_Document = doc;
		}

		void Show( UpdaterNotifyWindowItem item )
		{
			Init();

			if( FALSE == item.CheckShowNotifyWindow(m_Document) )
			{
				return;
			}

			m_Document->SetShowNotifyWindow(TRUE);
			if( FALSE == CreateNotifyWindow(item, m_Document) )
			{
				m_Document->SetShowNotifyWindow(FALSE);
				return;
			}

			if (m_window.IsWindow() == TRUE)
			{
				CRect rcClient;
				m_window.GetClientRect(&rcClient);

				//TrayIconHandler::CreateInstacne()->TryShow();

				m_Alpha = 0;

				m_rootwindow->SetAlpha(m_Alpha);
				m_window.SetWindowPos(HWND_TOPMOST, -rcClient.Width()-10, 0, 0, 0, SWP_NOSIZE | SWP_NOREDRAW | SWP_NOACTIVATE);
				m_window.RedrawWindow();
			}

			::SetTimer(this->m_hWnd, WM_USER + 1, 100, NULL);
		}

		void Cancel()
		{
			m_mouseHover = FALSE;

			KillTimer(WM_USER+1);

			if( NULL == m_Document )
				return;

			if( NULL == m_DataContext )
				return;

			HWND tmpHwnd = m_window.m_hWnd;
			IDataContext *tmpDataContext = m_DataContext;
			m_DataContext = NULL;

			if( ::IsWindow(m_window) )
			{
				m_window.ShowWindow(SW_HIDE);
				m_window = NULL;
			}
			else
				m_window = NULL;

			//TrayIconHandler::CreateInstacne()->TryHide();

			PostMessage(WM_HIDE_NOTIFYWINDOW, (WPARAM)tmpHwnd, (LPARAM)tmpDataContext);
		}

	private:
		BOOL GetMoniterRect( RECT &source, RECT &monitorRect, DWORD dwFlag = MONITOR_DEFAULTTONEAREST);
		BOOL GetPrimaryMoniterRect( RECT &monitorRect );
		CPoint GetCalculateWindowPosition( UINT taskbarType, int w, int h, RECT current, RECT taskbar );

		LRESULT OnHide(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
		{
			HWND tmpHwnd = (HWND)wParam;
			IDataContext *tmpDataContext = (IDataContext *)lParam;

			if( ::IsWindow(tmpHwnd) )
			{
				::DestroyWindow(tmpHwnd);
			}

			if( m_Document ){
				m_Document->SetShowNotifyWindow(FALSE);
			}

			return TRUE;
		}

		BOOL CreateNotifyWindow(UpdaterNotifyWindowItem item, UpdaterDocument *doc);

		int IsAlphaMode() 
		{
			if( m_timer_count < 11 )
				return 1;
			else if( m_timer_count > 39 )
				return -1;

			return 0;
		}

		void CalcuAlpha()
		{
			int m_adder = IsAlphaMode() * 20;
			m_Alpha += m_adder;
			
			if( m_adder == 0 || m_Alpha > 254 )
				m_Alpha = 254;
			else if( m_Alpha < 0 )
				m_Alpha = 0;
		}

		LRESULT OnTimer(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
		{
			if( !m_mouseHover )
				m_timer_count++;

			CalcuAlpha();
			Effect();

			if (IsFinished())
			{
				Cancel();
			}

			return TRUE;
		}
	BOOL IsFinished()
	{
		if ( m_timer_count > 50 )
		{
			return TRUE;
		}

		return FALSE;
	}

	private:
		void Init()
		{
			m_Alpha = 0;
			m_timer_count = 0;
//			m_ShowingTime = 5;
		}

		void Effect()
		{
			if (m_rootwindow->m_hWndBase != NULL &&
				::IsWindow(m_rootwindow->m_hWndBase) == TRUE &&
				m_window.m_hWnd != NULL &&
				::IsWindow(m_window.m_hWnd) == TRUE )
			{
				m_rootwindow->SetAlpha(m_Alpha);
				m_window.RedrawWindow();
				m_window.SetWindowPos(HWND_TOPMOST, m_Pos.x, m_Pos.y, 0, 0, SWP_NOSIZE|SWP_NOREDRAW|SWP_NOACTIVATE);
			}
			

		}

	private:
		CRootWindow*		m_rootwindow;
		CWindow				m_window;
		UpdaterDocument*	m_Document;
		SimpleMapDataContext*		m_DataContext;
		CPoint				m_Pos;
		int					m_Alpha;
		int					m_timer_count;
		BOOL				m_mouseHover;
	};
};